@@ -557,7 +557,7 @@ static int imap_getauth(int sock, struct query *ctl, char *greeting)
 	    }
 	}
     } else {
-	if (strstr(capabilities, "STARTTLS") && outlevel >= O_VERBOSE) {
+	if (!ctl->use_ssl && strstr(capabilities, "STARTTLS") && outlevel >= O_VERBOSE) {
 	    report(stdout, GT_("%s: WARNING: server offered STARTTLS but sslproto '' given.\n"), commonname);
 	}
     }
