@@ -35,8 +35,8 @@
    especially those whose name start with YY_ or yy_.  They are
    private implementation details that can be changed or removed.  */
 
-#ifndef YY_YY_RCFILE_Y_H_INCLUDED
-# define YY_YY_RCFILE_Y_H_INCLUDED
+#ifndef YY_YY__RCFILE_Y_H_INCLUDED
+# define YY_YY__RCFILE_Y_H_INCLUDED
 /* Debug traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 1
@@ -261,7 +261,7 @@ union YYSTYPE
   int number;
   char *sval;
 
-#line 265 "rcfile_y.h"
+#line 265 "../rcfile_y.h"
 
 };
 typedef union YYSTYPE YYSTYPE;
@@ -276,4 +276,4 @@ extern YYSTYPE yylval;
 int yyparse (void);
 
 
-#endif /* !YY_YY_RCFILE_Y_H_INCLUDED  */
+#endif /* !YY_YY__RCFILE_Y_H_INCLUDED  */
