@@ -414,24 +414,33 @@ va_dcl {
 #endif
 
 #ifdef USING_WOLFSSL
-# if LIBWOLFSSL_VERSION_HEX < 0x05005001L
-#  error "FAILED - wolfSSL MUST be at least version 5.5.1. You have " LIBWOLFSSL_VERSION_STRING "."
+# if LIBWOLFSSL_VERSION_HEX < 0x05006002L
+#  error "FAILED - wolfSSL MUST be at least version 5.6.2. You have " LIBWOLFSSL_VERSION_STRING "."
 # endif
-# if LIBWOLFSSL_VERSION_HEX < 0x05005003L
-#  pragma message "WARNING - wolfSSL SHOULD be at least version 5.5.3. You have " LIBWOLFSSL_VERSION_STRING "."
+# if LIBWOLFSSL_VERSION_HEX < 0x05006006L
+#  pragma message "WARNING - wolfSSL SHOULD be at least version 5.6.6. You have " LIBWOLFSSL_VERSION_STRING "."
 # endif
 #else /* !USING_WOLFSSL */
-#define fm_MIN_OPENSSL_VER 0x1000206fL /* 1.0.2f */
-# if OPENSSL_VERSION_NUMBER <  0x1010111fL
-#  pragma message "WARNING - OpenSSL 1.m.nx SHOULD be at least release version 1.1.1q, using " OPENSSL_VERSION_TEXT "."
+#define fm_MIN_OPENSSL_VER 0x1000215fL /* 1.0.2u */
+# if OPENSSL_VERSION_NUMBER <  0x1010115fL
+#  pragma message "WARNING - OpenSSL 1.m.nx SHOULD be at least release version 1.1.1u, using " OPENSSL_VERSION_TEXT "."
 # endif                     /* 0xMNN00PPSL */
-# if OPENSSL_VERSION_NUMBER >= 0x30000000L
-#  if OPENSSL_VERSION_NUMBER < 0x30000070L
-#   pragma message "WARNING - OpenSSL 3.m.n SHOULD be at least release version 3.0.7, using " OPENSSL_VERSION_TEXT "."
+/* do not warn about OpenSSL 3.2.0, the 3.2.1 fix is of low priority */
+# if OPENSSL_VERSION_NUMBER >= 0x30100000L && OPENSSL_VERSION_NUMBER < 0x30200000L
+#  if OPENSSL_VERSION_NUMBER < 0x30100040L
+#   pragma message "WARNING - OpenSSL 3.1.n SHOULD be at least release version 3.1.4, using " OPENSSL_VERSION_TEXT "."
 #  endif
 # endif                     /* 0xMNN00PPSL */
+# if OPENSSL_VERSION_NUMBER >= 0x30000000L && OPENSSL_VERSION_NUMBER < 0x30100000L
+#  if OPENSSL_VERSION_NUMBER < 0x30000090L
+#   pragma message "WARNING - OpenSSL 3.0.n SHOULD be at least release version 3.0.9, using " OPENSSL_VERSION_TEXT "."
+#  endif
+# endif                     /* 0xMNN00PPSL */
+#  if OPENSSL_VERSION_NUMBER < 0x30000000L
+#   pragma message "WARNING - OpenSSL before 3.0.0 is end-of-life and unsupported, using " OPENSSL_VERSION_TEXT "."
+#  endif
 # if OPENSSL_VERSION_NUMBER < fm_MIN_OPENSSL_VER
-#  error Your OpenSSL version must be at least 1.0.2f release. Older OpenSSL versions are unsupported.
+#  error Your OpenSSL version MUST be at least 1.0.2u release. Older OpenSSL versions are unsupported.
 # else /* OpenSSL too old */
 /*
 #define __fm_ossl_ver(x) #x
